<?php
/**
 * @file
 * NCIP Connection object
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

require_once('NCIPHTTPClient.php');

class NCIPConnection {
  const NCIP_INITIATING_CONNECTION = 0x0001;
  const NCIP_RESPONDING_CONNECTION = 0x0002; // Don't use yet

  const NCIP_HTTP_CONNECTION = 'http';
  const NCIP_HTTPS_CONNECTION = 'https'; // Don't use yet
  const NCIP_TCP_CONNECTION = 'tcp'; // Don't use yet

  const NCIP_STATELESS = 0x0000;
  const NCIP_IDLE_STATE = 0x0001;
  const NCIP_WAITING_STATE = 0x0002;
  const NCIP_PROCESSING_STATE = 0x0003;

  private $connection_id;

  // General information
  private $type;
  private $protocol;
  private $application;
  private $state;

  // Connection information
  private $host;
  private $port;
  private $path;

  public $timeout = 20; // Request timeout
  public $session = array();
  public $cookies = array();

  // Remote application information
  private $to_system_id = array();
  private $to_agency_id = array();

  // Basic connection settings
  public $use_session = TRUE;
  public $use_cookies = TRUE;

  // Message stack
  private $messages = array();

  private $errormsg;

  private $rawResponse = FALSE;

  /**
   * NCIPConnection Constructor
   */
  public function __construct(NCIPApplication $application, $connection_id) {
    $this->connection_id = $connection_id;
    $this->application = $application;
    $namespace = $this->application->get_namespace();
    $sql = "SELECT connection_id, type, protocol, state, host, port, path,
              timeout, session, cookies, to_system_id, to_agency_id, use_session,
              use_cookies
            FROM {ncip_connection}
            WHERE connection_id = %d AND application = '%s'";
    $connection = db_fetch_object(db_query($sql, $connection_id, $namespace));
    if ($connection) {
      $this->type = $connection->type;
      $this->protocol = $connection->protocol;
      $this->state = $connection->state;
      $this->host = $connection->host;
      $this->port = $connection->port;
      $this->path = $connection->path;
      $this->timeout = $connection->timeout;
      $this->session = unserialize($connection->session);
      $this->cookies = unserialize($connection->cookies);
      $this->to_system_id = unserialize($connection->to_system_id);
      $this->to_agency_id = unserialize($connection->to_agency_id);
      $this->use_session = $connection->use_session;
      $this->use_cookies = $connection->use_cookies;
      $this->update_state(NCIPConnection::NCIP_STATELESS);
    }
    else {
      drupal_set_message(t('Unexpected error. Cannot create NCIP connection.'), 'error');
    }
  }

  public function get_url() {
    return $this->protocol . '://' . $this->host . ':' . $this->port . $this->path;
  }

  /**
   * NCIPConnection destructor, may be useful for any future TCP implementation
   */
  public function __destruct() {
    // TODO: instead of desctuct we should use something different thing, because in Ajax calls it cased an error
    // this line is just a dirty hack. The problem is, that an update do some error, which
    // calls watchdog, but without user id, and the triggered error message in HTML format
    // is concatenated to the output, which is the result of the Ajax reponse.
    xc_log_info('ncip', 'fake');
    $this->disconnect();
    $this->update();
  }

  private function _func_if_exists($function, $args) {
    if (method_exists($this, $function[1])) {
      return call_user_func_array($function, $args);
    }
    elseif (function_exists($function[1])) {
      return call_user_func_array($function[1], $args);
    }
  }

  /**
   * Establish a network connection to the host.
   *
   * Note: NCIPConnection object must have first been created from an
   * NCIPApplication object.
   */
  public function connect() {
    $this->update_state(NCIPConnection::NCIP_IDLE_STATE);
    $function = array($this, '_ncip_' . $this->protocol . '_connect');
    $args = func_get_args();
    return $this->_func_if_exists($function, $args);
  }

  /**
   * Finish/end network connection
   *
   * Note: NCIPConnection object will continue to persist until it is destroyed
   * from its related NCIPApplication object, or until its current state timeout
   * for inactivity is reached
   */
  public function disconnect() {
    $this->update_state(NCIPConnection::NCIP_STATELESS);
    $function = array($this, '_ncip_' . $this->protocol . '_disconnect');
    $args = func_get_args();
    return $this->_func_if_exists($function, $args);
  }

  /**
   * Updates all properties of the NCIP connection, saves all values to the
   * database
   */
  public function update() {
    $sql = "UPDATE {ncip_connection}
            SET type = %d, protocol = '%s', application = '%s', state = %d,
              host = '%s', port = '%s', path = '%s', timeout = %d,
              timestamp = %d, session = '%s', cookies = '%s',
              to_system_id = '%s', to_agency_id = '%s', use_session = %d,
              use_cookies = %d
            WHERE connection_id = %d AND application = '%s'";
    db_query($sql, $this->type, $this->protocol,
      $this->application->get_namespace(), $this->state, $this->host,
      $this->port, $this->path, $this->timeout, time(),
      serialize($this->session), serialize($this->cookies),
      serialize($this->to_system_id), serialize($this->to_agency_id),
      $this->use_session, $this->use_cookies, $this->connection_id,
      $this->application->get_namespace());
  }

  /**
   * Updates the NCIP connection timestamp to track connection activity, saves
   * value to the database
   */
  public function update_timestamp() {
    $sql = "UPDATE {ncip_connection}
            SET timestamp = %d
            WHERE connection_id = %d AND application = '%s'";
    db_query($sql, time(), $this->connection_id, $this->application->get_namespace());
  }

  /**
   * Updates the NCIP connection state, saves value to the database
   */
  private function update_state($state) {
    $sql = "UPDATE {ncip_connection}
            SET state = %d, timestamp = %d
            WHERE connection_id = %d AND application = '%s'";
    db_query($sql, $this->state, time(), $this->connection_id,
      $this->application->get_namespace());
  }

  /**
   * Sends an NCIP message object over the connection
   *
   * @param $message
   *    NCIPMessage object
   * @return
   *    TRUE if NCIP message was sent or FALSE
   */
  public function send(NCIPMessage $message) {
    switch ($this->type) {
      case NCIPConnection::NCIP_INITIATING_CONNECTION:
        if ($this->state == NCIPConnection::NCIP_IDLE_STATE) {
          $this->update_state(NCIPConnection::NCIP_WAITING_STATE);
        }
        break;
      case NCIPConnection::NCIP_RESPONDING_CONNECTION:
        if ($this->state == NCIPConnection::NCIP_PROCESSING_STATE) {
          $this->update_state(NCIPConnection::NCIP_IDLE_STATE);
        }
        break;
    }
    // _ncip_http_send()
    // _ncip_https_send()
    // _ncip_tcp_send()
    $function = array($this, '_ncip_' . $this->protocol . '_send');
    $args = func_get_args();
    return $this->_func_if_exists($function, $args);
  }

  /**
   * Receives an incoming NCIP message over the connection
   *
   * @return
   *    NCIPMessage object from or NULL
   */
  public function receive() {
    switch ($this->type) {
      case NCIPConnection::NCIP_INITIATING_CONNECTION:
        if ($this->state == NCIPConnection::NCIP_WAITING_STATE) {
          $this->update_state(NCIPConnection::NCIP_IDLE_STATE);
        }
        break;
      case NCIPConnection::NCIP_RESPONDING_CONNECTION:
        if ($this->state == NCIPConnection::NCIP_IDLE_STATE) {
          $this->update_state(NCIPConnection::NCIP_PROCESSING_STATE);
        }
        break;
    }
    $function = array($this, '_ncip_' . $this->protocol . '_receive');
    $args = func_get_args();
    return $this->_func_if_exists($function, $args);
  }

  /** Getters and Setters **/
  public function set_to_system_id($scheme, $value) {
    $this->to_system_id = array('scheme' => $scheme, 'value' => $value);
  }

  public function get_to_system_id() {
    return $this->to_system_id;
  }

  public function set_to_agency_id($scheme, $value) {
    $this->to_agency_id = array('scheme' => $scheme, 'value' => $value);
  }

  public function get_to_agency_id() {
    return $this->to_agency_id;
  }

  public function get_connection_id() {
    return $this->connection_id;
  }

  public function get_type() {
    return $this->type;
  }

  public function get_application() {
    return $this->application;
  }

  public function get_protocol() {
    return $this->protocol;
  }

  public function get_timestamp() {
    $sql = "SELECT timestamp
            FROM {ncip_connection}
            WHERE connection_id = %d AND application = '%s'";
    return db_result(db_query($sql, $this->connection_id, $this->application->get_namespace()));
  }

  public function get_curr_state() {
    return $this->state;
  }

  public function get_curr_state_time() {
    return time() - $this->get_timestamp();
  }

  /** Supported protocol implementations **/
  private function &_ncip_http_connect($connect = TRUE) {
    static $http;

    if (is_null($http) && $connect) {
      $http['client'] = new NCIPHTTPClient($this->host, $this->port);
      $http['client']->cookie_host = $this->host;
      $http['client']->debug_on = variable_get('ncip_debugging', 0);
      $http['client']->use_gzip = FALSE;
      $http['client']->timeout = $this->timeout;
      $http['client']->content_type = 'application/xml; charset="utf-8"';
      if ($this->use_cookies) {
        $http['client']->set_cookies($this->cookies);
      }
    }

    return $http;
  }

  private function _ncip_http_disconnect() {
    // Retrieve static HTTPClient object from _ncip_http_connect()
    $http = &$this->_ncip_http_connect(FALSE);
    unset($http);
  }

  private function _ncip_http_send(NCIPMessage $message) {
    // Retrieve static HTTPClient object from _ncip_http_connect()
    $http = &$this->_ncip_http_connect();

    // Send cookies to HTTP client
    $http['client']->set_cookies($this->cookies);

    // POST message to server and receive XML response or return FALSE
    if (!$http['client']->post($this->path, $message->to_xml())) {
      $this->errormsg = $http['client']->errormsg;
      return FALSE;
    }

    // Parse into an NCIPMessage object and push into message stack
    $this->rawResponse = $http['client']->get_content();
    $response = NCIPMessage::from_xml($this->rawResponse);
    array_push($this->messages, $response);

    // After the result get cookies from HTTP client
    $this->cookies = $http['client']->get_cookies();

    // Send was successful, then return TRUE
    return TRUE;
  }

  private function _ncip_http_receive() {
    // Pop NCIPMessage object from message stack
    return array_shift($this->messages);
  }

  /** Currently unsupported protocol implementations **/
  private function _ncip_https_connect() {} // Currently not supported

  private function _ncip_https_disconnect() {} // Currently not supported

  private function _ncip_https_send() {} // Currently not supported

  private function _ncip_https_receive() {} // Currently not supported

  private function _ncip_tcp_connect() {} // Currently not supported

  private function _ncip_tcp_disconnect() {} // Currently not supported

  private function _ncip_tcp_send() {} // Currently not supported

  private function _ncip_tcp_receive() {} // Currently not supported

  public function get_errormsg() {
    return $this->errormsg;
  }

  public function getRawResponse() {
    return $this->rawResponse;
  }

}