<?php
/**
 * @file
 * Modified Version of Simon Willison’s NCIPHTTPClient (http://simon.incutio.com)
 * Manual (Albiet some modifications): http://scripts.incutio.com/NCIPHTTPClient
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

class NCIPHTTPClient {
  // Request vars
  public $host;
  public $port;
  public $path;
  public $method;
  public $postdata = '';
  public $cookies = array();
  public $referer;
  public $accept = 'text/xml,application/xml,application/xhtml+xml,text/html,text/plain,image/png,image/jpeg,image/gif,*/*';
  public $accept_encoding = 'gzip';
  public $accept_language = 'en-us';
  public $content_type = 'application/x-www-form-urlencoded';
  public $user_agent = 'eXtensible Catalog PHP HTTPClient';

  // Options
  public $timeout = 20;
  public $use_gzip = TRUE;

  // If TRUE, received cookies are placed in the $this->cookies array ready for
  // the next request
  // Note: This currently ignores the cookie path (and time) completely. Time
  // is not important, but path could possibly lead to security problems.
  public $persist_cookies = TRUE;

  // For each request, sends path of last request as referer
  public $persist_referers = TRUE;
  public $debug_on = FALSE;

  // Automatically redirect if Location or URI header is found
  public $handle_redirects = TRUE;
  public $max_redirects = 5;

  // If TRUE, stops receiving once headers have been read.
  public $headers_only = FALSE;

  // Basic authorization variables
  public $username;
  public $password;

  // Response vars
  public $status;
  public $headers = array();
  public $content = '';
  public $errormsg;

  // Tracker variables
  public $redirect_count = 0;
  public $cookie_host = '';

  public function __construct($host, $port = 80) {
    $this->host = $host;
    $this->port = $port;
  }

  public function get($path, $data = FALSE, $headers = array()) {
    $this->path = $path;
    $this->method = 'GET';
    if ($data) {
      $this->path .= '?'.$this->build_query_string($data);
    }
    return $this->do_request($headers);
  }

  public function post($path, $data = '', $headers = array()) {
    $this->path = $path;
    $this->method = 'POST';
    $this->postdata = $this->build_query_string($data);
    return $this->do_request($headers);
  }

  private function build_query_string($data) {
    $querystring = '';
    if (is_array($data)) {
      // Change data in to postable data
      foreach ($data as $key => $val) {
        if (is_array($val)) {
          foreach ($val as $val2) {
            $querystring .= urlencode($key).'='.urlencode($val2).'&';
          }
        }
        else {
          $querystring .= urlencode($key).'='.urlencode($val).'&';
        }
      }
      $querystring = drupal_substr($querystring, 0, -1); // Eliminate unnecessary &
    }
    else {
      $querystring = $data;
    }
    return $querystring;
  }

  private function do_request($headers = array()) {
    // Performs the actual HTTP request, returning TRUE or FALSE depending on outcome
    if ($this->timeout === FALSE) {
      $fp = @fsockopen($this->host, $this->port, $errno, $errstr);
    }
    else {
      $fp = @fsockopen($this->host, $this->port, $errno, $errstr, $this->timeout);
    }
    if (!$fp) {
      // Set error message

      switch ($errno) {
        case -3:
          $this->errormsg = 'Socket creation failed (-3)'; break;
        case -4:
          $this->errormsg = 'DNS lookup failure (-4)'; break;
        case -5:
          $this->errormsg = 'Connection refused or timed out (-5)'; break;
        default:
          $this->errormsg = 'Connection failed (' . $errno . ')'; break;
      }
      $this->errormsg .= ' ' . $errstr;
      $this->debug($this->errormsg);
      return FALSE;
    }

    if ($this->timeout !== FALSE) {
      socket_set_timeout($fp, $this->timeout);
    }
    $request = $this->build_request($headers);
    $this->debug('Request', $request);
    fwrite($fp, $request);

    // Reset all the variables that should not persist between requests
    $this->headers = array();
    $this->content = '';
    $this->errormsg = '';

    // Set a couple of flags
    $inHeaders = TRUE;
    $atStart = TRUE;

    $response = '';
    // Now start reading back the response
    while (!feof($fp)) {
      $line = fgets($fp, 4096);

      // Response for debuging purposes
      $response .= $line;

      if ($atStart) {
        // Deal with first line of returned data
        $atStart = FALSE;
        if (!preg_match('/HTTP\/(\\d\\.\\d)\\s*(\\d+)\\s*(.*)/', $line, $m)) {
          $status = stream_get_meta_data($fp);
          if ($status['timed_out']) {
            $this->errormsg = t('Connection timed out, request exceed %timeout', array('%timeout' => $this->timeout));
          }
          else {
            $this->errormsg = t('Status code line invalid: "!line"', array(!line => htmlentities($line)));
          }
          $this->debug($this->errormsg);
          return FALSE;
        }
        $http_version = $m[1]; // not used
        $this->status = $m[2];
        $status_string = $m[3]; // not used
        $this->debug(trim($line));
        continue;
      }
      if ($inHeaders) {
        if (trim($line) == '') {
          $inHeaders = FALSE;
          $this->debug('Received Headers', $this->headers);
          if ($this->headers_only) {
            break; // Skip the rest of the input
          }
          continue;
        }
        if (!preg_match('/([^:]+):\\s*(.*)/', $line, $m)) {
          // Skip to the next header
          continue;
        }
        $key = strtolower(trim($m[1]));
        $val = trim($m[2]);

        // Deal with the possibility of multiple headers of same name
        if (isset($this->headers[$key])) {
          if (is_array($this->headers[$key])) {
            $this->headers[$key][] = $val;
          }
          else {
            $this->headers[$key] = array($this->headers[$key], $val);
          }
        }
        else {
          $this->headers[$key] = $val;
        }
        continue;
      }
      // We're not in the headers, so append the line to the contents
      $this->content .= $line;
    }
    $status = stream_get_meta_data($fp);
    fclose($fp);

    // If data is compressed, uncompress it
    if (isset($this->headers['content-encoding']) && $this->headers['content-encoding'] == 'gzip') {
      $this->debug('Content is gzip encoded, unzipping it');
      $this->content = drupal_substr($this->content, 10); // See http://www.php.net/manual/en/function.gzencode.php
      $this->content = gzinflate($this->content);
    }

    $this->debug('Response', $response);

    // If $persist_cookies, deal with any cookies
    if ($this->persist_cookies
        && isset($this->headers['set-cookie'])
        && $this->host == $this->cookie_host) {
      $cookies = $this->headers['set-cookie'];
      if (!is_array($cookies)) {
        $cookies = array($cookies);
      }
      foreach ($cookies as $cookie) {
        if (preg_match('/([^=]+)=([^;]+);/', $cookie, $m)) {
          $this->cookies[$m[1]] = $m[2];
        }
      }
      // Record domain of cookies for security reasons
      $this->cookie_host = $this->host;
    }

    // If $persist_referers, set the referer ready for the next request
    if ($this->persist_referers) {
      $this->debug('Persisting referer: ' . $this->get_request_url());
      $this->referer = $this->get_request_url();
    }

    // Finally, if handle_redirects and a redirect is sent, do that
    if ($this->handle_redirects) {
      if (++$this->redirect_count >= $this->max_redirects) {
        $this->errormsg = 'Number of redirects exceeded maximum (' . $this->max_redirects . ')';
        $this->debug($this->errormsg);
        $this->redirect_count = 0;
        return FALSE;
      }
      $location = isset($this->headers['location']) ? $this->headers['location'] : '';
      $uri = isset($this->headers['uri']) ? $this->headers['uri'] : '';
      if ($location || $uri) {
        $url = parse_url($location.$uri);
        // This will FAIL if redirect is to a different site
        return $this->get($url['path']);
      }
    }
    return TRUE;
  }

  private function build_request() {
    $headers = array();
    $headers[] = $this->method . ' '  . $this->path . '} HTTP/1.0'; // Using 1.1 leads to all manner of problems, such as "chunked" encoding
    $headers[] = 'Host: ' . $this->host . ':' . $this->port;
    $headers[] = 'User-Agent: ' . $this->user_agent;
    $headers[] = 'Accept: ' . $this->accept;
    if ($this->use_gzip) {
      $headers[] = 'Accept-encoding: ' . $this->accept_encoding;
    }
    $headers[] = 'Accept-language: ' . $this->accept_language;
    if ($this->referer) {
      $headers[] = 'Referer: ' . $this->referer;
    }
    // Cookies
    if ($this->cookies) {
      $cookie = 'Cookie: ';
      foreach ($this->cookies as $key => $value) {
        $_key = (string) $key;
        $_value = is_array($value) ? (string) current($value) : (string) $value;
        $cookie .= $_key . '=' . $_value . '; ';
      }
      $headers[] = $cookie;
    }
    // Basic authentication
    if ($this->username && $this->password) {
      $headers[] = 'Authorization: BASIC '
                 . base64_encode($this->username . ':' . $this->password);
    }
    // If this is a POST, set the content type and length
    if ($this->postdata) {
      $headers[] = 'Content-Type: ' . $this->content_type;
      $headers[] = 'Content-Length: ' . strlen($this->postdata);
    }
    $request = implode("\r\n", $headers) . "\r\n\r\n" . $this->postdata;
    return $request;
  }

  public function get_status() {
    return $this->status;
  }

  public function get_content() {
    return $this->content;
  }

  public function get_headers() {
    return $this->headers;
  }

  public function get_header($header) {
    $header = strtolower($header);
    if (isset($this->headers[$header])) {
      return $this->headers[$header];
    }
    else {
      return FALSE;
    }
  }

  public function get_error() {
    return $this->errormsg;
  }

  public function get_cookies() {
    return $this->cookies;
  }

  public function get_request_url() {
    $url = 'http://'.$this->host;
    if ($this->port != 80) {
      $url .= ':'.$this->port;
    }
    $url .= $this->path;
    return $url;
  }

  // Setter methods
  public function set_user_agent($string) {
    $this->user_agent = $string;
  }

  public function set_authorization($username, $password) {
    $this->username = $username;
    $this->password = $password;
  }

  public function set_cookie($name, $value) {
    $this->cookies = array_merge($this->cookies, array($name => $value));
  }

  public function set_cookies($array) {
    $this->cookies = array_merge($this->cookies, $array);
  }

  // Option setting methods
  public function use_gzip($boolean) {
    $this->use_gzip = $boolean;
  }

  public function set_persist_cookies($boolean) {
    $this->persist_cookies = $boolean;
  }

  public function set_persist_referers($boolean) {
    $this->persist_referers = $boolean;
  }

  public function set_handle_redirects($boolean) {
    $this->handle_redirects = $boolean;
  }

  public function set_max_redirects($num) {
    $this->max_redirects = $num;
  }

  public function set_headers_only($boolean) {
    $this->headers_only = $boolean;
  }

  public function set_debug($boolean) {
    $this->debug_on = $boolean;
  }

  // "Quick" static methods
  public static function quick_get($url) {
    $bits = parse_url($url);
    $host = $bits['host'];
    $port = isset($bits['port']) ? $bits['port'] : 80;
    $path = isset($bits['path']) ? $bits['path'] : '/';
    if (isset($bits['query'])) {
      $path .= '?'.$bits['query'];
    }
    $client = new NCIPHTTPClient($host, $port);
    if (!$client->get($path)) {
      return FALSE;
    }
    else {
      return $client->get_content();
    }
  }

  public static function quick_post($url, $data) {
    $bits = parse_url($url);
    $host = $bits['host'];
    $port = isset($bits['port']) ? $bits['port'] : 80;
    $path = isset($bits['path']) ? $bits['path'] : '/';
    $client = new NCIPHTTPClient($host, $port);
    if (!$client->post($path, $data)) {
      return FALSE;
    }
    else {
      return $client->get_content();
    }
  }

  function debug($message, $object = FALSE, $watchdog = TRUE) {
    if ($this->debug_on) {
      $watchdog ? watchdog('http client', $message, NULL)
                : drupal_set_message('<strong>NCIP HTTP Client:</strong> '.$message);
      if ($object) {
        $content = (string) print_r($object, TRUE);
        $content_pre = '<pre>'.htmlentities($content).'</pre>';
        $watchdog ? watchdog('http client', $content_pre, NULL)
                  : drupal_set_message($content_pre);
      }
    }
  }
}