<?php
/**
 * @file
 * NCIP Message object
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

class NCIPMessage {
  const NCIP_INITIATION_MESSAGE = 0x0001;
  const NCIP_RESPONSE_MESSAGE = 0x0002;

  const NCIP_XML_DTD = 0x0001;
  const NCIP_XML_SCHEMA = 0x0001;

  // General information
  private $type;
  private $connection;

  public $sent = FALSE;
  public $received = FALSE;

  // Message structure
  public $service;
  public $header;
  public $message = array();
  public $problem = array();

  private $xml_structure;
  private $version_string;

  /**
   * Load message from XML string
   *
   * @param $connection
   *    Connection where message originated or should be directed
   * @param $xml
   *    XML string
   * @return
   *    NCIPMessage object
   */
  public static function from_xml($xml, $connection = NULL) {
    // TODO: find a better error handling
    $xml = preg_replace('/&(?!#x?\d+;|[a-zA-Z]+;)/', "&amp;$1", $xml);
    $dom = @DOMDocument::loadXML($xml);
    return NCIPMessage::from_dom($dom, $connection);
  }

  /**
   * Load message from XML DOM object
   *
   * @param $connection
   *    Connection where message originated or should be directed
   * @param $dom
   *    XML DOM object
   * @return
   *    NCIPMessage object
   */
  public static function from_dom($dom, $connection = NULL) {
    if ($dom instanceof DOMDocument) {
      $message_node = $dom->getElementsByTagName('NCIPMessage')->item(0);
      $message = NCIPMessage::_dom_to_array($message_node, 'NCIPMessage');
      $service = key($message);

      return new NCIPMessage($connection, $service, $message);
    }
    else {
      return NULL;
    }
  }

  /**
   * NCIPMessage constructor
   *
   * @param $connection
   *    Connection where message should originate
   * @param $service
   *    NCIP service
   * @param $message
   *    Message array structure
   */
  public function __construct($connection, $service = '', $message = array(),
      $xml_structure = NCIPMessage::NCIP_XML_DTD) {
    $this->connection = $connection;
    $this->service = $service;
    $this->message = $message;
    $this->xml_structure = $xml_structure;

    // Place service and headers into message and merge with message parameter
    $this->init_message();

    // Check for and report problems
    if ($this->has_problem()) {
      watchdog('problem', '<pre>%text</pre>', array(
        '%text' => $this->_array_to_xml(
          $this->message[$this->service]['Problem'],
          'Problem')
        )
      );
    }
  }

  /**
   * Initializes message by adding header and version attribute information
   */
  private function init_message() {
    if ($this->connection instanceof NCIPConnection) {
      $this->init_from_connection();
    }
    else {
      // Set properties of the message object if it is does not originate from
      // an NCIPConnection or NCIPApplication object, for example, if it is
      // parsed from an incoming XML
      if (preg_match('/Response$/', $this->service)) {
        $this->set_type(NCIPMessage::NCIP_RESPONSE_MESSAGE);
        $this->header = 'ResponseHeader';
      }
      else {
        $this->set_type(NCIPMessage::NCIP_INITIATION_MESSAGE);
        $this->header = 'InitiationHeader';
      }
    }

    // Determine the required DTD or XML Schema version attribute
    switch ($this->xml_structure) {
      case NCIPMessage::NCIP_XML_SCHEMA:
        $this->version_string = 'http://www.niso.org/ncip/v1_0/imp1/xtd/ncip_v1_0.xsd';
        break;
      case NCIPMessage::NCIP_XML_DTD:
      default:
        $this->version_string = 'http://www.niso.org/ncip/v1_0/imp1/dtd/ncip_v1_0.dtd';
        break;
    }

    // Place message into message array
    $this->problem = array();
    if (isset($this->message[$this->service]['Problem'])
        && is_array($this->message[$this->service]['Problem'])) {
      $this->problem = $this->message[$this->service]['Problem'];
    }
  }

  private function init_from_connection() {
    $connection = $this->connection;
    $application = $this->connection->get_application();

    // Determine type and header from NCIP connection
    switch ($connection->get_type()) {
      case NCIPConnection::NCIP_RESPONDING_CONNECTION:
        $this->type = NCIPMessage::NCIP_RESPONSE_MESSAGE;
        $this->header = 'ResponseHeader';
        break;

      case NCIPConnection::NCIP_INITIATING_CONNECTION:
      default:
        $this->type = NCIPMessage::NCIP_INITIATION_MESSAGE;
        $this->header = 'InitiationHeader';
        break;
    }

    // Insert header element into message
    $header = &$this->insert_element($this->header);

    // Gather data elements for header
    $from_agency_id = $application->get_from_agency_id();
    $from_agency_authentication = $application->get_from_agency_authentication();
    $from_system_id = $application->get_from_system_id();
    $from_system_authentication = $application->get_from_system_authentication();
    $to_system_id = $connection->get_to_system_id();
    $to_agency_id = $connection->get_to_agency_id();

    if ($this->type == NCIPMessage::NCIP_INITIATION_MESSAGE) {
      $on_behalf_of_agency = $application->get_on_behalf_of_agency();
      $application_profile_type = $application->get_application_profile_type();
    }

    // Insert FromAgencyId
    $this->insert_scheme(
      array($this->header, 'FromAgencyId', 'UniqueAgencyId'),
      $from_agency_id['scheme'],
      $from_agency_id['value']
    );

    // Insert ToAgencyId
    $this->insert_scheme(
      array($this->header, 'ToAgencyId', 'UniqueAgencyId'),
      $to_agency_id['scheme'],
      $to_agency_id['value']
    );

    // Insert FromSystemId
    if (!empty($from_system_id['scheme']) && !empty($from_system_id['value'])) {
      $this->insert_scheme(
        array($this->header, 'FromSystemId'),
        $from_system_id['scheme'],
        $from_system_id['value']
      );
    }

    // Insert ToSystemId
    if (!empty($to_system_id['scheme']) && !empty($to_system_id['value'])) {
      $this->insert_scheme(
        array($this->header, 'ToSystemId'),
        $to_system_id['scheme'],
        $to_system_id['value']
      );
    }

   // Insert FromAgencyAuthentication
    if (!empty($from_agency_authentication)) {
      $this->insert_string(
        array($this->header, 'FromAgencyAuthentication'),
        $from_agency_authentication);
    }

    // Insert FromSystemAuthentication
    if (!empty($from_system_authentication)) {
      $this->insert_string(
        array($this->header, 'FromSystemAuthentication'),
        $from_system_authentication);
    }

    // Insert OnBehalfOfAgency
    if (!empty($on_behalf_of_agency['scheme'])
        && !empty($on_behalf_of_agency['value'])) {
      $this->insert_scheme(
        array($this->header, 'OnBehalfOfAgency', 'UniqueAgencyId'),
        $on_behalf_of_agency['scheme'],
        $on_behalf_of_agency['value']
      );
    }

    // Insert ApplicationProfileType
    if (!empty($application_profile_type['scheme'])
        && !empty($application_profile_type['value'])) {
      $this->insert_scheme(
        array($this->header, 'ApplicationProfileType'),
        $to_system_id['scheme'],
        $to_system_id['value']
      );
    }
  }

  /**
   * Check whether there is a problem with the NCIP message
   */
  public function has_problem() {
     return empty($this->problem) ? FALSE : TRUE;
  }

  /** Getters and setters **/
  public function set_type($type) {
    if (empty($type)) {
      $this->type = $type;
    }
  }

  public function set_xml_structure($xml_structure) {
    if (empty($this->xml_structure)) {
      $this->xml_structure = $xml_structure;
    }
  }

  public function get_xml_structure() {
    return $this->xml_structure;
  }

  /**
   * Insert a data element (and, optionally, its value) into the NCIP message
   *
   * This function will call one of the following functions, depending on the
   * type of element in at the last position in the $path:
   *  - insert_boolean($path, $boolean)
   *  - insert_datetime($path, $datetime)
   *  - insert_integer($path, $integer)
   *  - insret_non_negative_integer($path, $non_negative_integer)
   *  - insert_positive_integer($path, $positive_integer)
   *  - insert_string($path, $string)
   *  - insert_scheme($path, $scheme, $value)
   *
   * See each respective function for more information.
   *
   * @param $path
   *    An array representing the path to the element, wihtout including the
   *    service element for example:
   *    array('FirstChildUnderService', 'ChildOfFirstChild');
   *
   *    A good example is with the init_message() function which inserts
   *    elements for the header using this array to insert the FromAgencyId
   *    data element:
   *    array('InitiationHeader', 'FromAgencyId')
   *
   *    In some cases, there may be more than one element with the same node
   *    name at a certain level in the path. To distinglish between them, use
   *    the following format:
   *    array(
   *      array('name' => 'SecondRepeatingChild', 'delta' => 1),
   *      'ChildOfFirstRepeatingChild',
   *      array(
   *        'name' => 'FirstRepeatingChildOfChildOfSecondRepeatingChild',
   *        'delta' => 0
   *      )
   *    );
   *
   * @param $value
   *    The value to be placed in that element. If it is a scheme, this should
   *    be an array in the form of: array('scheme' => $scheme, 'value' => $value)
   * @return
   *    The element with value
   */
  public function &insert($path, $value = NULL) {
    $path = is_array($path) ? $path : func_get_args();

    // Get data element from application
    $application = $this->connection->get_application();
    $data_elements = $application->get_data_elements();
    $data_element = $data_elements[key(end($path))];

    // Based on the type of NCIP element call the right function
    switch ($data_element['type']) {
      case NCIP_BOOLEAN:
        return $this->insert_boolean($path, $value);
        break;
      case NCIP_DATETIME:
        return $this->insert_datetime($path, $value);
        break;
      case NCIP_INTEGER:
        return $this->insert_integer($path, $value);
        break;
      case NCIP_NON_NEGATIVE_INTEGER:
        return $this->insert_non_negative_integer($path, $value);
        break;
      case NCIP_POSITIVE_INTEGER:
        return $this->insert_positive_integer($path, $value);
        break;
      case NCIP_STRING:
        return $this->insert_string($path, $value);
        break;
      case NCIP_OPEN_SCHEME:
      case NCIP_CLOSED_SCHEME:
        return $this->insert_scheme($path, $value['scheme'], $value['value']);
        break;
      case NCIP_COMPLEX_ELEMENT:
      case NCIP_EMPTY_ELEMENT:
      default:
        return $this->insert_element($path);
        break;
    }
  }

  /**
   * Create an heirarchial path to an element within the NCIP message.
   *
   * A depth level is created as the function iterates through each element
   * in the $path array.
   *
   * @param $path
   *    An array representing the ath to the element, for example:
   *    array('FirstChild', 'ChildOfFirstChild')
   * @return
   *    A reference to the last element in the path
   */
  public function &insert_element($path) {
    if (empty($this->service)) {
      return FALSE;
    }

    // Create all elements, if necessary
    $node = &$this->message[$this->service];
    $path = is_array($path) ? $path : func_get_args();
    foreach ($path as $element) {
      $node = is_array($node) ? $node : array();
      if (is_array($element)) {
        $tag = array_shift($element);
        $delta = array_shift($element);
        if (!empty($node[$tag]) && !isset($node[$tag][0])) {
          $node[$tag] = array($node[$tag]);
        }
        $node = &$node[$tag][$delta];
      }
      elseif (isset($node[$element][0])) {
        $node = &$node[$element][];
      }
      else {
        $node = &$node[$element];
      }
    }
    return $node;
  }

  /**
   * Insert a boolean value into an NCIP message element. Will convert the
   * variable to boolean, if possible.
   *
   * See insert($path, $value) for more information.
   */
  public function &insert_boolean($path, $boolean) {
    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -1);
    $node = &$this->insert_element($path);

    // Insert boolean
    $node = (boolean) $boolean;
    return $node;
  }

  /**
   * Insert a datetime value into an NCIP message element. Will parse a
   * timestamp from the variable, if possible, and convert it to ISO-8601
   * format.
   *
   * See insert($path, $value) for more information.
   */
  public function &insert_datetime($path, $datetime) {
    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -1);
    $node = &$this->insert_element($path);

    // Insert dateTime
    $timestamp = is_numeric($datetime) ? $datetime : @strtotime($datetime);
    $node = @date(DATE_ISO8601, $timestamp);
    return $node;
  }

  /**
   * Insert an integer value into an NCIP message element. Will convert the
   * variable to int, if possible.
   *
   * See insert($path, $value) for more information.
   */
  public function &insert_integer($path, $integer) {
    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -1);
    $node = &$this->insert_element($path);

    // Insert integer
    $node = (int) $integer;
    return $node;
  }

  /**
   * Insert a non-negative integer value into an NCIP message element. Will
   * convert the variable to int, if possible, and return FALSE if the
   * interger result is negative.
   *
   * See insert($path, $value) for more information.
   */
  public function &insert_non_negative_integer($path, $non_negative_integer) {
    if (((int) $non_negative_integer) < 0) {
      return FALSE;
    }

    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -1);
    $node = &$this->insert_element($path);

    // Insert integer
    $node = (int) $non_negative_integer;
    return $node;
  }

  /**
   * Insert a positive integer value into an NCIP message element. Will
   * convert the variable to int, if possible, and return FALSE if the
   * interger result is not positive.
   *
   * See insert($path, $value) for more information.
   */
  public function &insert_positive_integer($path, $positive_integer) {
    if (((int) $positive_integer) < 0) {
      return FALSE;
    }

    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -1);
    $node = &$this->insert_element($path);

    // Insert integer
    $node = (int) $positive_integer;
    return $node;
  }

  /**
   * Insert a string integer value into an NCIP message element. Will
   * convert the variable to string, if possible.
   *
   * See insert($path, $value) for more information.
   */
  public function &insert_string($path, $string) {
    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -1);
    $node = &$this->insert_element($path);

    // Insert integer
    $node = (string) $string;
    return $node;
  }

  /**
   * Insert an open or closed scheme value into an NCIP message element.
   * Requires both $scheme and $path parameters.
   *
   * See insert($path, $value) for more information.
   *
   * @param $path (Array)
   *   The elemeent’s path
   * @param $scheme (String)
   *   The scheme’s URL
   * @param $value (String)
   *   A value
   * @return unknown_type

   */
  public function insert_scheme($path, $scheme, $value) {
    // Create all elements, if necessary
    $args = func_get_args();
    $path = is_array($path) ? $path : array_splice($args, -2);
    $node = &$this->insert_element($path);

    // Insert integer
    $node = array('Scheme' => (string) $scheme, 'Value' => (string) $value);
    return $node;
  }

  /**
   * Transforms NCIPMessage object into a valid NCIP XML string
   */
  public function to_xml() {
    $xml = '<?xml version="1.0" encoding="UTF-8"?>';
    $xml .= '<!DOCTYPE NCIPMessage PUBLIC "-//NISO//NCIP DTD Version 1//EN"
             "http://www.niso.org/ncip/v1_0/imp1/dtd/ncip_v1_0.dtd">';
    $xml .= '<NCIPMessage version="' . $this->version_string . '">';
    $xml .= NCIPMessage::_array_to_xml($this->message);
    $xml .= '</NCIPMessage>';
    return $xml;
  }

  /**
   * Transforms NCIPMessage object into a valid NCIP DOMDocument object
   */
  public function to_dom() {
    $implementation = new DOMImplementation();
    $dtd = $implementation->createDocumentType('NCIPMessage',
      '-//NISO//NCIP DTD Version 1//EN',
      'http://www.niso.org/ncip/v1_0/imp1/dtd/ncip_v1_0.dtd');
    $dom = $implementation->createDocument('', '', $dtd);
    $dom->version = '1.0';
    $dom->encoding = 'UTF-8';
    $root = $dom->createElement('NCIPMessage');
    $root->setAttribute('version', $this->version_string);
    $dom->appendChild($root);
    NCIPMessage::_array_to_dom($this->message, $dom, $root);
    return $dom;
  }

  /**
   * Private function used to transform NCIPMessage object into XML string
   */
  private static function _array_to_xml($array, $tag = '') {
    $xml = array();

    foreach ($array as $key => $value) {
      $tag = is_numeric($key) ? $tag : $key;
      if (is_array($value) && isset($value[0])) {
        $xml[] = NCIPMessage::_array_to_xml($value, $key);
      }
      else {
        $xml[] = '<' . $tag . '>';
        if (is_array($value)) {
          $xml[] = NCIPMessage::_array_to_xml($value);
        }
        else {
          $xml[] = htmlspecialchars($value, ENT_COMPAT, 'UTF-8');
        }
        $xml[] = '</' . $tag . '>';
      }
    }
    return implode('', $xml);
  }

  /**
   * Private function used to transform NCIPMessage object into a DOMDocument
   * object
   */
  private static function _array_to_dom($array, $dom, $node, $tag = '') {
    foreach ($array as $key => $value) {
      $tag = is_numeric($key) ? $tag : $key;
      if (is_array($value) && isset($value[0])) {
        NCIPMessage::_array_to_dom($value, $dom, $node, $key);
      }
      else {
        $child = $dom->createElement($tag, (is_array($value) ? NULL : $value));
        $node->appendChild($child);
        if (is_array($value)) {
          NCIPMessage::_array_to_dom($value, $dom, $child);
        }
      }
    }
  }

  /**
   * Private function used to load an XML string or DOMDocument object
   * into an NCIPMessage object
   */
  private static function _dom_to_array($node, $node_name) {
    $array = array();
    $node_name = $node->nodeName;

    if ($node->childNodes) {
      foreach ($node->childNodes as $child) {
        if ($child->nodeType == XML_ELEMENT_NODE) {
          $continue = TRUE;
          if ($siblings = $node->childNodes) {
            $count = 0;
            foreach ($siblings as $sibling) {
              if ($sibling->nodeName == $child->nodeName) {
                $count++;
              }
            }
            if ($count > 1) {
              $array[$child->nodeName][] = NCIPMessage::_dom_to_array($child,
                $node_name);
              $continue = FALSE;
            }
          }
          if ($continue) {
            $array[$child->nodeName] = NCIPMessage::_dom_to_array($child,
              $node_name);
          }
        }
        elseif ($child->nodeType == XML_TEXT_NODE
            && trim($child->nodeValue) != "") {
          $array = preg_replace('/\n/', ' ', trim($child->nodeValue));
        }
      }
    }
    return $array;
  }
}
