<?php
/**
 * @file
 *
 * The original version of this class was the Omeka’s OAI-PMH plugin made by
 * Center for History and New Media. The current version contains some fragments
 * from the first and second versions of that library, but it is rewritten in
 * almost all parts.
 */

class OAIHarvester {

  const NAMESPACE_URI = 'http://www.openarchives.org/OAI/2.0/';

  private $parser;
  private $baseUrl;
  private $requestVerb;
  private $requestArguments;
  private $http_cache;
  private $httpCode;
  private $error;
  private $content;
  private $hasFetched = FALSE;
  private $statistics = array();

  public function __construct($requestVerb = 'ListRecords', $baseUrl = '', $requestArguments = array(), $http_cache = FALSE) {
    $this->requestVerb = $requestVerb;
    $this->baseUrl = $baseUrl;
    $this->requestArguments = $requestArguments;
    $this->http_cache = $http_cache;
  }

  public function fetchContent($type = 'url', $filename = '') {
    $t0 = microtime(TRUE);
    $this->content = '';
    if ($type == 'url') {
      $this->fetchHttpContent();
    }
    elseif ($type == 'file') {
      $this->fetchFileContent($filename);
    }
    $this->statistics['fetch']['all'] = microtime(TRUE) - $t0;
  }

  public function fetchFileContent($filename) {
    try {
      $this->content = file_get_contents($filename);
      $this->httpCode = 200;
    }
    catch (Exception $e) {
      xc_log_error('harvester', var_export($e, TRUE));
      $this->error = $e->getMessage();
      $this->httpCode = 0;
    }
    $this->hasFetched = TRUE;
  }

  public function fetchHttpContent() {
    $t0 = microtime(TRUE);
    $requestUrl = $this->baseUrl . '?verb=' . $this->requestVerb;
    if (count($this->requestArguments)) {
      $requestUrl .= '&' . http_build_query($this->requestArguments, NULL, '&');
    }

    if ($this->http_cache !== FALSE && !empty($this->http_cache)) {
      $cache_file = $this->http_cache . '/' . md5($requestUrl);
    }

    if (isset($cache_file) && file_exists($cache_file)) {
      try {
        $this->content = file_get_contents($cache_file);
      }
      catch (Exception $e) {
        $this->error = $e->getMessage();
      }
      $this->statistics['fetch']['curl_init'] = microtime(TRUE) - $t0;
      $this->httpCode = 200;
    }
    else {
      $ch = curl_init("");
      curl_setopt($ch, CURLOPT_HEADER, 0); //1
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($ch, CURLOPT_NOPROGRESS, 1);
      curl_setopt($ch, CURLOPT_USERAGENT, 'Omeka-XC OAI harvester');
      curl_setopt($ch, CURLOPT_ENCODING, "gzip,deflate");
      curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
      if (substr($requestUrl, 0, 5) == 'https') {
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
      }
      curl_setopt($ch, CURLOPT_URL, $requestUrl);
      $this->statistics['fetch']['curl_init'] = microtime(TRUE) - $t0;
      $this->content = curl_exec($ch);
      $this->httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
      $download_size = curl_getinfo($ch, CURLINFO_SIZE_DOWNLOAD);

      if (isset($cache_file) && $this->httpCode == 200) {
        file_put_contents($cache_file, $this->content);

        // put content into the list
        $file_list = $this->http_cache . '/list.txt';
        $file_info = sprintf("%s\t%s\t%s\n", format_date(time(), 'custom', 'Y-m-d H:i:s'), md5($requestUrl), $requestUrl);
        file_put_contents($file_list, $file_info, FILE_APPEND);
        unset($list);
      }
      if (curl_errno($ch)) {
        $this->error = curl_error($ch);
      }
      curl_close($ch);
    }
    $this->hasFetched = TRUE;
  }

  public function processContent($type = 'dom') {
    if (!$this->hasFetched) {
      $this->fetchContent();
    }
    // IOAIParser is the parser interface
    require_once 'IOAIParser.php';
    switch ($type) {
      // regex-based parser
      case 'regex':
        require_once 'OAIRegexParser.php';
        $this->parser = new OAIRegexParser($this->content, $this->httpCode);
        break;
      // DOM-based parser
      case 'dom':
      default:
        require_once 'OAIDomParser.php';
        $this->parser = new OAIDomParser($this->content, $this->httpCode);
    }
  }

  /**
   * Validate the request verb.
   *
   * @param string $requestVerb
   *   The verb to request
   *
   * @return (boolean)
   */
  protected static function validateRequestVerb($requestVerb) {
    switch ($requestVerb) {
      case 'Identify':
      case 'ListMetadataFormats':
      case 'ListSets':
      case 'GetRecord':
      case 'ListIdentifiers':
      case 'ListRecords':
        return true;
        break;
    }
    throw new Exception('OAI-PMH parser error: Invalid request verb: ' . $requestVerb);
  }

  /**
   * Validate a URL.
   * @param $url
   * @return unknown_type
   */
  protected static function validateUrl($url) {
    if (!preg_match('/^http/', $url) || !@fopen($url, 'rb')) {
      throw new Exception(
        'OAI-PMH parser error: Could not validate or open provided URL.');
    }
    return true;
  }

  /**
   * Build the schema array containing all the names and namespace URIs of
   * the extended schema in the lib/Schema directory.
   */
  protected function _setSchemas() {
    foreach (new DirectoryIterator(OAIPMH_SCHEMA_DIRECTORY_PATH) as $item) {
      if ($item->isFile()) {
        include_once(OAIPMH_SCHEMA_DIRECTORY_PATH . DIRECTORY_SEPARATOR . $item);
        $schemaName   = drupal_substr($item, 0, strpos($item, '.'));
        $className    = 'Schema_' . $schemaName;
        $namespaceUri = call_user_func(array($className, 'getNamespaceUri'));
        $this->_schemas[] = array(
          'schemaName'   => $schemaName,
          'namespaceUri' => $namespaceUri
        );
      }
    }
  }

  public function getSets() {
    if ($this->requestVerb != 'ListSets') {
      return FALSE;
    }
    if (!$this->hasFetched) {
      $this->fetchContent();
    }
    // print $this->content;
    $items = array();
    do {
      $oaipmh = new SimpleXMLIterator($this->content);
      if (isset($oaipmh->ListSets)) {
        $sets = $oaipmh->ListSets->set;
        foreach ($sets as $set) {
          $item = (object)array(
            'set_spec' => (string) $set->setSpec,
            'display_name' => (string) $set->setName,
            'description' => '',
          );
          if (isset($set->setDescription)) {
            $descriptions = array();
            $dom = dom_import_simplexml($set->setDescription);
            foreach ($dom->childNodes as $childNode) {
              if ($childNode->nodeType == XML_ELEMENT_NODE) {
                $sescriptions[] = $childNode->nodeValue;
              }
            }
            $item->description = join(' ', $sescriptions);
          }
          $items[] = $item;
        }
        $continue = FALSE;
        if ($oaipmh->ListSets->resumptionToken) {
          $token = $oaipmh->ListSets->resumptionToken;
          if ((string)$token != '') {
            $this->requestArguments = array('resumptionToken' => (string)$oaipmh->ListSets->resumptionToken);
            print_r($this->requestArguments);
            $this->fetchContent();
            if (!$this->isErrorResponse()) {
              $continue = TRUE;
            }
          }
        }
      }
    }
    while ($continue);

    return $items;
  }

  public function getIdentify() {
    if ($this->requestVerb != 'Identify') {
      return FALSE;
    }
    if (!$this->hasFetched) {
      $this->fetchContent();
    }
    $oaipmh = new SimpleXMLIterator($this->content);
    $identify = array();
    $identify['repositoryName'] = (string)$oaipmh->Identify->repositoryName;
    $identify['baseUrl'] = (string)$oaipmh->Identify->baseURL;
    $identify['protocolVersion'] = (string)$oaipmh->Identify->protocolVersion;
    foreach ($oaipmh->Identify->adminEmail as $email) {
      $identify['adminEmail'][] = (string)$email;
    }

    $identify['earliestDatestamp'] = (string)$oaipmh->Identify->earliestDatestamp;
    $identify['deletedRecord'] = (string)$oaipmh->Identify->deletedRecord;
    $identify['granularity'] = (string)$oaipmh->Identify->granularity;
    $identify['compressions'] = array();
    foreach ($oaipmh->Identify->compression as $compression) {
      $identify['compressions'][] = (string)$compression;
    }

    foreach ($oaipmh->Identify->description as $description) {
      $dom = dom_import_simplexml($description);
      foreach ($dom->childNodes as $child) {
        if ($child->nodeType == XML_ELEMENT_NODE) {
          $identify['description'][] = array(
            'namespaceURI' => $child->namespaceURI,
            'childNode' => $child
          );
        }
      }
    }

    return $identify;
  }

  public function getMetadataFormats() {
    if ($this->requestVerb != 'ListMetadataFormats') {
      return FALSE;
    }
    if (!$this->hasFetched) {
      $this->fetchContent();
    }
    $formats = array();
    $oaipmh = new SimpleXMLIterator($this->content);
    if (isset($oaipmh->ListMetadataFormats)) {
      $metadataFormats = $oaipmh->ListMetadataFormats->metadataFormat;
      foreach ($metadataFormats as $metadataFormat) {
        $formats[] = (object)array(
          'name' => (string) $metadataFormat->metadataPrefix,
          'schema_location' => (string) $metadataFormat->schema,
          'namespace' => (string) $metadataFormat->metadataNamespace
        );
      }
    }

    // todo: resumptionToken
    return $formats;
  }

  public function isErrorResponse() {
    if (!$domDocument = @DOMDocument::loadXML($this->content)) {
      throw new Exception($this->content);
    }

    if ($domDocument->getElementsByTagNameNS(self::NAMESPACE_URI, 'error')->item(0) === NULL) {
      return FALSE;
    }
    else {
      $error = $domDocument->getElementsByTagNameNS(self::NAMESPACE_URI, 'error');
      $this->error = array(
        'code' => $error->item(0)->getAttribute('code'),
        'text' => $error->item(0)->nodeValue
      );
      return TRUE;
    }
  }

  public function getBaseUrl() {
    return $this->baseUrl;
  }

  public function getContent() {
    return $this->content;
  }

  public function getHttpCode() {
    return $this->httpCode;
  }

  function getStatistics($type = 'all') {
    switch ($type) {
      case 'all':
        return $this->statistics; break;
      case 'parser':
        return $this->parser->getStatistics(); break;
      case 'fetch':
        return $this->statistics['fetch']['all'];
      case 'parse_response':
        return $this->parser->getStatistics('response'); break;
      case 'parse_records':
        return $this->parser->getStatistics('records'); break;
      default:
        return;
    }
  }

  /* WRAPPER FUNCTIONS FOR PARSER INTERFACE FUNCTIONS */

  public function hasErrors() {
    return $this->parser->hasErrors();
  }

  /** Get all errors */
  public function listErrors() {
    return $this->parser->listErrors();
  }

  /** Has records? */
  public function hasRecords() {
    return $this->parser->hasRecords();
  }

  /** Get number of records */
  public function getRecordCount() {
    return $this->parser->getRecordCount();
  }

  /** Get next record */
  public function getNextRecord() {
    return $this->parser->getNextRecord();
  }

  /** Has resumptionToken? */
  public function hasResumptionToken() {
    return $this->parser->hasResumptionToken();
  }

  /** Get the resumptionToken */
  public function getResumptionToken() {
    return $this->parser->getResumptionToken();
  }

}