<?php
/**
 * @file
 * Regular expression based implementation of the OAI-PMH parser.
 *
 * The XML response is parsed with regular expressions.
 *
 * @author Király Péter <kirunews@gmail.com>
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

class OAIRegexParser implements IOAIParser {

  private $rawContent;
  private $raw_records;
  private $raw_resumption;
  private $httpCode;

  private $errorMsgs        = array();
  private $statistics       = array();
  private $hasErrors        = FALSE;
  private $hasListRecords   = FALSE;
  private $hasRecords       = FALSE;
  private $recordCount      = 0;
  private $records          = NULL;
  private $recordCounter    = -1;
  private $resumptionToken;
  private $hasResumptionToken = FALSE;
  private $need_debug       = FALSE;

  function __construct($content, $httpCode = 200) {
    $this->httpCode = $httpCode;
    $this->rawContent = $content;
    $t0 = microtime(TRUE);
    $this->process();
    $this->statistics['response'] = microtime(TRUE) - $t0;
  }

  /* PUBLIC FUNCTIONS */
  function setHttpCode($httpCode) {
    $this->httpCode = $httpCode;
  }

  function getHttpCode() {
    return $this->httpCode;
  }

  function process() {
    if ($this->httpCode == 200) {
      $this->rawContent = preg_replace('/<!--.*?-->/', '', $this->rawContent);
      $this->_processContent();
    }
    else {
      $this->errorMsg[] = $this->rawContent;
    }
  }

  function processFile($fileName) {
    $this->rawContent = file_get_contents($fileName);
    $this->_processContent();
  }

  function hasErrors() {
    return $this->hasErrors || ((int)$this->httpCode != 200 || !empty($this->errorMsgs));
  }

  /**
   * @return (Array)
   *   The list of server provided error messages. Each error has an 'attributes' array, and a 'text'
   */
  function listErrors() {
    return $this->errorMsgs;
  }

  function hasErrorMsg() {
    return empty($this->errorMsg);
  }

  function getErrorMsg() {
    return $this->errorMsg;
  }

  function hasRecords() {
    return $this->hasRecords;
  }

  function getRecordCount() {
    return $this->recordCount;
  }

  function getNextRecord() {
    if (++$this->recordCounter < $this->recordCount) {
      $xml = $this->records[$this->recordCounter];
      $t0 = microtime(TRUE);
      try {
        $record = $this->parseRecord($xml);
      }
      catch (Exception $e) {
        xc_log_error("parser", $e->getMessage());
      }
      $this->statistics['records'] += (microtime(TRUE) - $t0);
      return $record;
    }
    else {
      return FALSE;
    }
  }

  function hasResumptionToken() {
    return $this->hasResumptionToken;
  }

  function getResumptionToken() {
    return $this->resumptionToken;
  }

  function getStatistics($type = 'all') {
    switch ($type) {
      case 'all':
        return $this->statistics; break;
      case 'response':
        return $this->statistics['response'];
      case 'records':
        return $this->statistics['records'];
      default:
        return;
    }
  }

  /* PRIVATE FUNCTIONS */
  private function _processContent() {
    if (strstr($this->rawContent, '<error') && !strstr($this->rawContent, '<ListRecords>')) {
      $text = $this->rawContent;
    }
    else {
      $text = $this->rawContent;
      if (!$this->need_debug) {
        $this->rawContent = '';
      }
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^<\?xml[^<>]+\?>/", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no sheabang');
      $this->errorMsgs[] = 'no sheabang';
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<OAI-PMH\b[^<>]+>/s", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no &lt;OAI-PMH&gt;');
      $this->errorMsgs[] = 'no OAI-PMH';
      return;
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<responseDate>(.*?)<\/responseDate>/s", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no &lt;responseDate&gt;');
      $this->errorMsgs[] = 'no responseDate';
      return;
    }

    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<request(?: [^<>]+)?>(.*?)<\/request>/s", "", $text);
    if ($size == drupal_strlen($text)) {
      xc_log_error('regex', 'no &lt;request&gt; element in XML response: '
        . htmlentities(substr($text, 0, 50)) . '&hellip;');
      $this->errorMsgs[] = 'no request';
      return;
    }

    $this->hasListRecords = FALSE;
    $this->hasErrors = FALSE;
    $size = drupal_strlen($text);
    $text = preg_replace("/^\s*<ListRecords>/", "", $text);
    if ($size == drupal_strlen($text)) {
      $errors = '';
      $size = drupal_strlen($text);
      $text = preg_replace_callback(
        "/^\s*<error(?: code=([\'\"])(.*)([\'\"]))?>(.*)<\/error>/",
        array($this, 'setErrors'),
        $text
      );
      if ($size == drupal_strlen($text)) {
        xc_log_error('regex', 'no ListRecords and error');
        $this->errorMsgs[] = 'no ListRecords and error';
      }
      else {
        $this->hasErrors = TRUE;
      }
    }
    else {
      $this->hasListRecords = TRUE;
    }
    if (!$this->hasErrors && !$this->need_debug) {
      $this->rawContent = '';
    }

    if ($this->hasListRecords) {
      $resumption = '';
      $records    = '';
      $is_changed_size = TRUE;
      while (!preg_match("/^\s*<\/ListRecords>/", $text) && $is_changed_size) {
        $size = drupal_strlen($text);
        $text = preg_replace_callback("/^\s*(<record>.*<\/record>)/s", array($this, 'setRecords'), $text);

        if ($size != drupal_strlen($text)) {
          $this->hasRecords = TRUE;
          $is_changed_size = TRUE;
          // raw_records is initialized by setRecords
          $mod_records = str_replace('<record>', '<#######><record>', $this->raw_records);
          $this->records = preg_split("/<#######>/", $mod_records, -1, PREG_SPLIT_NO_EMPTY);
          $this->recordCount = count($this->records);
          $mod_records = '';

          $size = drupal_strlen($text);
          $text = preg_replace_callback("/^\s*(<resumptionToken([^<>]*)>(.*?)<\/resumptionToken>|<resumptionToken([^<>]*)\s*\/\s*>)/",
            array($this, 'setResumption'), $text);
          if ($size != drupal_strlen($text)) {
            $is_changed_size = TRUE;
            // resumption comes from setResumption
            //echo 'recCount: ', $this->recordCount, ", resumption: ", $this->resumptionToken['text'], "\n";
          }
          else {
            $is_changed_size = FALSE;
            //$this->errorMsgs[] = 'no resumptionToken';
          }
        }
        else {
          $is_changed_size = FALSE;
          $this->errorMsgs[] = 'no records';
        }
      } // process inside ListRecords

      $size = drupal_strlen($text);
      $text = preg_replace('/^\s*<\/ListRecords>/', '', $text);
      if ($size == drupal_strlen($text)) {
        $this->errorMsgs[] = 'no /ListRecords';
      }
    } // has ListRecords
    else {
      $this->errorMsgs[] = 'no ListRecords';
    }

    if ($this->hasListRecords || $this->hasErrors) {
      $size = drupal_strlen($text);
      $text = preg_replace("/^\s*<\/OAI-PMH>/", '', $text);
      if ($size == drupal_strlen($text)) {
        $this->errorMsgs[] = 'no /OAI-PMH';
      }
    } // has ListRecords or error
  }

  /**
   * Save the error message come from OAI-PMH data provider
   *
   * @param $matches (Array)
   *   matches[2]: the error code (optional)
   *   matches[4]: the error message
   * @return (string)
   *   An empty string to replace the error message
   */
  private function setErrors($matches) {
    $this->errorMsgs[] = t('OAI-PMH data provider sent an error message.')
     . ' '
     . (isset($matches[2]) ? $matches[2] . ': ' : '') . $matches[4];
    return "";
  }

  private function setRecords($matches) {
    $this->raw_records = $matches[1];
    return "";
  }

  private function setResumption($matches) {
    $xml = simplexml_load_string($matches[1]);
    $this->resumptionToken = array(
      'attributes' => array(
        'completeListSize' => (int)$xml['completeListSize'],
        'cursor'           => (int)$xml['cursor'],
        'expirationDate'   => (string)$xml['expirationDate'],
      ),
      'text'               => (string)$xml,
    );
    $this->hasResumptionToken = !empty($this->resumptionToken['text']);
    return "";
  }

  public function setDebug($need_debug = FALSE) {
    $this->need_debug = $need_debug;
  }

  function parseRecord($record_str) {
    $record = new SimpleXMLElement($record_str);
    $specs = array();
    if (isset($record->header->setSpec)) {
      foreach ($record->header->setSpec as $spec) {
        $specs[] = (string)$spec;
      }
    }
    $metadata = array();
    if (isset($record->metadata)) {
      $dom = dom_import_simplexml($record->metadata);
      if ($dom) {
        foreach ($dom->childNodes as $childNode) {
          if ($childNode->nodeType == XML_ELEMENT_NODE) {
            $metadata = array(
              'namespaceURI' => preg_replace('/\/$/', '', $childNode->namespaceURI),
              'childNode'    => $childNode,
            );
            break;
          }
        }
      }
    }

    return array(
      'header' => array(
        '@status' => isset($record->header->attributes()->status) ? $record->header->attributes()->status : FALSE,
        'identifier' => (string)$record->header->identifier[0],
        'datestamp' => (string)$record->header->datestamp[0],
        'setSpec' => $specs
      ),
      'metadata' => $metadata,
      'about' => isset($record->abouts) ? $record->abouts : FALSE
    );
  }
}
